@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
 # Copyright 2006 Joe Wreschnig
+#           2021 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,10 +27,12 @@ class WAVEFile(AudioFile):
                 f = wave.open(h)
                 self["~#length"] = float(f.getnframes()) / f.getframerate()
                 self["~#channels"] = f.getnchannels()
+                self["~#samplerate"] = f.getframerate()
+                self["~#bitdepth"] = f.getsampwidth() * 8
         self.sanitize(filename)
 
-    def sanitize(self, filename):
-        super(WAVEFile, self).sanitize(filename)
+    def sanitize(self, filename=None):
+        super().sanitize(filename)
         self["title"] = fsn2text(os.path.splitext(
             os.path.basename(self["~filename"]))[0])
 
