@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
-#           2016 Ryan Dellenbaugh
-#           2017 Nick Boultbee
+#                2016 Ryan Dellenbaugh
+#           2017-2022 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +11,7 @@ import codecs
 import re
 
 from . import _match as match
-from ._match import ParseError
+from ._match import ParseError, Units
 from quodlibet.util import re_escape
 
 
@@ -31,7 +30,7 @@ TEXT = re.compile(r'[^,)]+')
 DATE = re.compile(r'\d{4}(-\d{1,2}(-\d{1,2})?)?')
 
 
-class QueryParser(object):
+class QueryParser:
     """Parse the input. One lookahead token, start symbol is Query."""
 
     def __init__(self, tokens, star=[]):
@@ -115,10 +114,10 @@ class QueryParser(object):
             return self.Intersection(self.Numcmp)
         elif self.accept('@'):
             return self.Extension()
+        index = self.index
         try:
             # Equals, NotEquals and Star can begin the same,
             # so try in order, backtracking on failure (with Star last)
-            index = self.index
             return self.Equals()
         except ParseError:
             self.index = index
@@ -184,7 +183,7 @@ class QueryParser(object):
             if self.accept(':'):
                 # time like 4:15
                 number2 = float(self.expect_re(DIGITS))
-                expr = match.NumexprNumber(60 * number + number2)
+                expr = match.NumexprNumber(60 * number + number2, units=Units.SECONDS)
             elif self.accept_re(WORD):
                 # Number with units like 7 minutes
                 expr = match.numexprUnit(number, self.last_match)
